﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using FDAL.Database;

namespace FDAL.Gen
{
    public class TEntityGenerator
    {   
        private TDatabaseExplorer explorer;

        public TEntityGenerator(SqlConnection conn)
        {
            this.explorer = new TDatabaseExplorer(conn);
        }

        public string GenerateEntity(TTable table)
        {
            string className = TNameConvert.GetPascalString(table.Name);
            TCodeWriter w = new TCodeWriter();
            w.WriteLineFormat("public class {0}", className);
            w.WriteLine("{");
            w.PushIdent();
            IDictionary<string, Type> columns = explorer.GetTableColumns(table.SafeName);
            foreach (string columnName in columns.Keys)
            {
                string propertyName = TNameConvert.GetPascalString(columnName);
                //string propertyName = columnName;
                Type columnType = columns[columnName];
                string columnTypeName = columnType.Name;
                if (columnType.IsValueType)
                {
                    columnTypeName += "?";
                }
                string propertyDef = string.Format("public {0} {1} {{ get; set; }}", columnTypeName, propertyName);
                w.WriteLine(propertyDef);
            }

            w.PopIdent();
            w.Write("}");

            return w.Code;
        }

        public string GenerateEntities(List<TTable> tables, string nameSpace)
        {
            TCodeWriter writer = new TCodeWriter();
            writer.WriteLine("// KDAL Generated Code");
            writer.WriteLineFormat(string.Format("//Date: {0}", DateTime.Now));
            writer.WriteLine();
            writer.WriteLine();
            writer.WriteLine("using System;");
            writer.WriteLine();
            writer.WriteLineFormat("namespace {0}", nameSpace);
            writer.WriteLine("{");
            writer.PushIdent();

            foreach(TTable table in tables)
            {
                string entity = GenerateEntity(table);
                writer.WriteLine(entity);
                writer.WriteLine();
            }
            writer.PopIdent();
            writer.Write("}");
            return writer.Code;
        }
    }
}
