﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;

namespace FDAL.Gen
{
    public static class TypeMapper
    {
        static IDictionary<SqlDbType, Type> typeMap = new Dictionary<SqlDbType, Type>();
        static IDictionary<Type, string> rtypeMap = new Dictionary<Type, string>();
        static IDictionary<Type, string> nativeMap = new Dictionary<Type, string>();

        static TypeMapper()
        {
            typeMap.Add(SqlDbType.BigInt, typeof(Int64));
            typeMap.Add(SqlDbType.Binary, typeof(Byte[]));
            typeMap.Add(SqlDbType.Bit, typeof(Boolean));
            typeMap.Add(SqlDbType.Char, typeof(String));
            typeMap.Add(SqlDbType.DateTime, typeof(DateTime));
            typeMap.Add(SqlDbType.Decimal, typeof(Decimal));
            typeMap.Add(SqlDbType.Float, typeof(Double));
            typeMap.Add(SqlDbType.Image, typeof(Byte[]));
            typeMap.Add(SqlDbType.Int, typeof(Int32));
            typeMap.Add(SqlDbType.Money, typeof(Decimal));
            typeMap.Add(SqlDbType.NChar, typeof(String));
            typeMap.Add(SqlDbType.NText, typeof(String));
            typeMap.Add(SqlDbType.NVarChar, typeof(String));
            typeMap.Add(SqlDbType.Real, typeof(Single));
            typeMap.Add(SqlDbType.UniqueIdentifier, typeof(Guid));
            typeMap.Add(SqlDbType.SmallDateTime, typeof(DateTime));
            typeMap.Add(SqlDbType.SmallInt, typeof(Int16));
            typeMap.Add(SqlDbType.SmallMoney, typeof(Decimal));
            typeMap.Add(SqlDbType.Text, typeof(String));
            typeMap.Add(SqlDbType.Timestamp, typeof(DateTime));
            typeMap.Add(SqlDbType.TinyInt, typeof(byte));
            typeMap.Add(SqlDbType.VarBinary, typeof(Byte[]));
            typeMap.Add(SqlDbType.VarChar, typeof(String));
            typeMap.Add(SqlDbType.Variant, typeof(Object));
            typeMap.Add(SqlDbType.Xml, typeof(String));
            typeMap.Add(SqlDbType.Udt, typeof(Object));
            typeMap.Add(SqlDbType.Structured, typeof(Object));
            typeMap.Add(SqlDbType.Date, typeof(DateTime));
            typeMap.Add(SqlDbType.Time, typeof(TimeSpan));
            typeMap.Add(SqlDbType.DateTime2, typeof(DateTime));
            typeMap.Add(SqlDbType.DateTimeOffset, typeof(DateTimeOffset));
            
            rtypeMap.Add(typeof(Int64), "BIGINT");
            rtypeMap.Add(typeof(Boolean), "BIT");
            rtypeMap.Add(typeof(String), "NVARCHAR(500)");
            rtypeMap.Add(typeof(DateTime), "DATETIME");
            rtypeMap.Add(typeof(Decimal), "DECIMAL");
            rtypeMap.Add(typeof(Double), "FLOAT");
            rtypeMap.Add(typeof(Byte[]), "VARBINARY(MAX)");
            rtypeMap.Add(typeof(Int32), "INT");
            rtypeMap.Add(typeof(Single), "REAL");
            rtypeMap.Add(typeof(Guid), "UNIQUEIDENTIFIER");
            rtypeMap.Add(typeof(Int16), "SMALLINT");
            rtypeMap.Add(typeof(byte), "TINYINT");
            rtypeMap.Add(typeof(TimeSpan), "TIME");
            rtypeMap.Add(typeof(DateTimeOffset), "DATETIMEOFFSET");
        }

        public static Type GetCLRType(SqlDbType sqlType)
        {
            return typeMap[sqlType];
        }

        public static string GetDBType(Type type)
        {
            return rtypeMap[type];
        }
    }
}
