﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Reflection;
using System.Text;

namespace FDAL
{
    public static class ReaderExtensions
    {
        public static bool HasColumn(this SqlDataReader reader, string columnName)
        {
            bool found = false;

            int i = 0;
            while (!found && i < reader.FieldCount)
            {
                found = columnName.Equals(reader.GetName(i), StringComparison.OrdinalIgnoreCase);
                i++;
            }
            return found;
        }

        public static List<object> MapToList(this SqlDataReader reader, Type itemType)
        {
            List<object> list = new List<object>();
            while (reader.Read())
            {
                object item = Activator.CreateInstance(itemType);
                foreach (PropertyInfo pi in itemType.GetProperties())
                {
                    string dbField = pi.Name.ToUpper();
                    if (reader.HasColumn(dbField))
                    {
                        if (reader[dbField].GetType() != typeof(DBNull))
                        {
                            pi.SetValue(item, reader[dbField], null);
                        }
                    }
                }
                list.Add(item);
            }
            return list;
        }
    }
}
