﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace FDAL
{
    public class TContext
    {
        private SqlConnection connection;
        private SqlTransaction transaction;


        public TContext(SqlConnection connection, SqlTransaction transaction)
        {
            this.connection = connection;
            this.transaction = transaction;
        }

        public TContext(SqlConnection connection) : this(connection, null) { }


        private SqlParameter getReturnParameter()
        {
            SqlParameter p = new SqlParameter();
            p.ParameterName = "RETURN_VALUE";
            p.Direction = ParameterDirection.ReturnValue;
            return p;
        }

        public TStoredProcResult ExecProc(string name, IEnumerable<SqlParameter> parameters, bool query, IEnumerable<Type> resultTypes)
        {
            using (SqlCommand command = new SqlCommand(name, connection))
            {
                TStoredProcResult result = new TStoredProcResult();
                command.CommandType = CommandType.StoredProcedure;
                if (transaction != null)
                {
                    command.Transaction = transaction;
                }

                // add return parameter
                command.Parameters.Add(getReturnParameter());

                if (parameters != null)
                {
                    command.Parameters.AddRange(parameters.ToArray<SqlParameter>());
                }

                if (!query)
                {
                    result.AffectedRows = command.ExecuteNonQuery();
                    result.ReturnedValue = command.Parameters[0].Value;
                }
                else
                {
                    SqlDataReader reader = null;
                    List<List<object>> lists = new List<List<object>>();
                    try
                    {
                        reader = command.ExecuteReader();
                        int i = 0;
                        do
                        {
                            Type itemType = resultTypes.ElementAtOrDefault<Type>(i);
                            if (itemType != null)
                            {
                                List<object> l = reader.MapToList(itemType);
                                lists.Add(l);
                                i++;
                            }
                        } while (reader.NextResult());
                        result.ResultSets = lists;
                    }
                    finally
                    {
                        if (reader != null)
                        {
                            reader.Close();
                            result.AffectedRows = reader.RecordsAffected;
                            result.ReturnedValue = command.Parameters[0].Value;
                        }
                    }
                }

                // add out paramaters
                foreach (SqlParameter p in command.Parameters)
                {
                    if (p.Direction == ParameterDirection.InputOutput || p.Direction == ParameterDirection.Output)
                    {
                        if (result.OutPramaters == null)
                        {
                            result.OutPramaters = new List<SqlParameter>();
                        }
                        result.OutPramaters.Add(p);
                    }
                }


                return result;
            }
        }

        public TStoredProcResult ExecProc(string name, IEnumerable<SqlParameter> parameters, params Type[] resultTypes)
        {
            if (resultTypes.Length > 0)
                return ExecProc(name, parameters, true, resultTypes);
            else
                return ExecProc(name, parameters, false, null);
        }


        public TStoredProcResult ExecNonQueryProc(string name, IEnumerable<SqlParameter> parameters)
        {
            return ExecProc(name, parameters, false, null);
        }

        public TStoredProcResult ExecQueryProc(string name, IEnumerable<SqlParameter> parameters, IEnumerable<Type> resultTypes)
        {
            return ExecProc(name, parameters, true, resultTypes);
        }
    }
}
