﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using FDAL.Database;

namespace FDAL
{
    public class TDatabaseExplorer
    {
        private SqlConnection conn;

        public TDatabaseExplorer(SqlConnection conn)
        {
            this.conn = conn;
        }
        public static bool TestConnection(string cn, out string errorMessage)
        {
            SqlConnection conn = null;
            errorMessage = null;
            bool result = false;
            try
            {
                conn = new SqlConnection(cn);
                conn.Open();
                result = true;
            }
            catch (Exception ex)
            {
                result = false;
                errorMessage = ex.Message;
            }
            finally
            {
                if (conn != null)
                {
                    conn.Close();
                }
            }

            return result;
        }

        public List<TDatabase> GetDatabases()
        {
            SqlCommand cmd = new SqlCommand();
            cmd.Connection = conn;
            cmd.CommandType = CommandType.Text;
            cmd.CommandText = StatementStore.GetInstance().GetStatement("GET_DATABASES");
            SqlDataReader rdr = cmd.ExecuteReader();
            List<TDatabase> dbList = new List<TDatabase>();
            while (rdr.Read())
            {
                TDatabase db = new TDatabase();
                db.Id = rdr.GetInt16(0);
                db.Name = rdr.GetString(1);
                dbList.Add(db);
            }
            rdr.Close();
            return dbList;
        }

        public List<TSchema> GetSchemas()
        {
            SqlCommand cmd = new SqlCommand();
            cmd.Connection = conn;
            cmd.CommandType = CommandType.Text;
            cmd.CommandText = StatementStore.GetInstance().GetStatement("GET_SCHEMAS");
            SqlDataReader rdr = cmd.ExecuteReader();
            List<TSchema> scList = new List<TSchema>();
            while (rdr.Read())
            {
                TSchema sc = new TSchema();
                sc.Id = rdr.GetInt16(0);
                sc.Name = rdr.GetString(1);
                scList.Add(sc);
            }
            rdr.Close();
            return scList;
        }

        public List<TTable> GetTables()
        {    
            SqlCommand cmd = new SqlCommand();
            cmd.Connection = conn;
            cmd.CommandType = CommandType.Text;
            cmd.CommandText = StatementStore.GetInstance().GetStatement("GET_TABLES");
            SqlDataReader rdr = cmd.ExecuteReader();
            List<TTable> tableList = new List<TTable>();
            while (rdr.Read())
            {
                TTable table = new TTable();
                table.Schema = rdr.GetString(0);
                table.Name = rdr.GetString(1);
                tableList.Add(table);
            }
            rdr.Close();
            return tableList;
        }
        
        public List<TStoredProcedure> GetStoredProcedures()
        {
            SqlCommand cmd = new SqlCommand
            {
                Connection = conn,
                CommandType = CommandType.Text,
                CommandText = StatementStore.GetInstance().GetStatement("GET_STORED_PROCEDURES")
            };
            SqlDataReader rdr = cmd.ExecuteReader();
            List<TStoredProcedure> spList = new List<TStoredProcedure>();
            while (rdr.Read())
            {
                TStoredProcedure sp = new TStoredProcedure();
                sp.Schema = rdr.GetString(0);
                sp.Name = rdr.GetString(1);
                spList.Add(sp);
            }
            rdr.Close();
            return spList;
        }

        public List<TStoredProcedure> GetStoredProceduresBySchema(string schemaName)
        {
            string s = StatementStore.GetInstance().GetStatement("GET_STORED_PROCEDURES_BY_SCHEMA");
            string sql = string.Format(s, schemaName);
            SqlCommand cmd = new SqlCommand
            {
                Connection = conn,
                CommandType = CommandType.Text,
                CommandText = sql
            };
            SqlDataReader rdr = cmd.ExecuteReader();
            List<TStoredProcedure> spList = new List<TStoredProcedure>();
            while (rdr.Read())
            {
                TStoredProcedure sp = new TStoredProcedure();
                sp.Schema = rdr.GetString(0);
                sp.Name = rdr.GetString(1);
                spList.Add(sp);
            }
            rdr.Close();
            return spList;
        }

        public SqlParameterCollection GetStoredProcParameters(TStoredProcedure storedProc)
        {
            SqlParameterCollection parameters;
            SqlCommand cmd = new SqlCommand
            {
                Connection = conn,
                CommandType = CommandType.StoredProcedure,
                CommandText = storedProc.Name
            };
            SqlCommandBuilder.DeriveParameters(cmd);
            parameters = cmd.Parameters;
            return parameters;
        }

        public List<TColumn> GetColumns(TTable table)
        {
                SqlCommand cmd = new SqlCommand();
                cmd.Connection = conn;
                cmd.CommandType = CommandType.Text;
                string s = StatementStore.GetInstance().GetStatement("GET_COLUMNS");
                string sql = string.Format(s, table.Schema, table.Name);
                cmd.CommandText = sql;
                SqlDataReader reader = cmd.ExecuteReader();
                List<TColumn> cols = reader.MapToList(typeof(TColumn)).OfType<TColumn>().ToList<TColumn>();
                reader.Close();
                return cols;
        }

        public TColumn GetColumn(TTable table, string columnName)
        {
            SqlCommand cmd = new SqlCommand();
            cmd.Connection = conn;
            cmd.CommandType = CommandType.Text;
            string s = StatementStore.GetInstance().GetStatement("GET_COLUMN");
            string sql = string.Format(s, table.Schema, table.Name, columnName);
            cmd.CommandText = sql;
            SqlDataReader reader = cmd.ExecuteReader();
            List<TColumn> cols = reader.MapToList(typeof(TColumn)).OfType<TColumn>().ToList<TColumn>();
            reader.Close();
            return cols.FirstOrDefault<TColumn>();
        }

        public IDictionary<string, Type> GetTableColumns(string table)
        {
            SqlCommand cmd = new SqlCommand();
            cmd.Connection = conn;
            cmd.CommandType = CommandType.Text;
            cmd.CommandText = "SET FMTONLY ON; SELECT * FROM " + table + "; SET FMTONLY OFF;";
            SqlDataReader rdr = cmd.ExecuteReader(CommandBehavior.KeyInfo);
            IDictionary<string, Type> columns = new Dictionary<string, Type>();
            DataTable dt = rdr.GetSchemaTable();

            foreach (DataRow row in dt.Rows)
            {
                string n = (string)row["ColumnName"];
                Type t = (Type)row["DataType"];
                if (!columns.ContainsKey(n))
                    columns.Add(n, t);
            }
            return columns;
        }

        public List<TConstraint> GetConstraints(TTable table)
        {
            SqlCommand cmd = new SqlCommand();
            cmd.Connection = conn;
            cmd.CommandType = CommandType.Text;
            string s = StatementStore.GetInstance().GetStatement("GET_CONSTRAINTS");
            string sql = string.Format(s, table.Schema, table.Name);
            cmd.CommandText = sql;
            List<TConstraint> constraintList = new List<TConstraint>();
            SqlDataReader rdr = cmd.ExecuteReader();
            while (rdr.Read())
            {
                TConstraint constraint = new TConstraint();
                constraint.Schema = rdr.GetString(0);
                constraint.Name = rdr.GetString(1);
                TTable ctable = new TTable();
                ctable.Schema = rdr.GetString(2);
                ctable.Name = rdr.GetString(3);
                constraint.Table = ctable;
                constraint.Type = rdr.GetString(4);
                constraint.Columns = GetConstraintColumns(constraint);
                constraintList.Add(constraint);
            }

            return constraintList;
        }

        public List<TColumn> GetConstraintColumns(TConstraint constraint)
        {
            SqlCommand cmd = new SqlCommand();
            cmd.Connection = conn;
            cmd.CommandType = CommandType.Text;
            string s = StatementStore.GetInstance().GetStatement("GET_CONSTRAINT_COLUMNS");
            string sql = string.Format(s, constraint.Schema, constraint.Name);
            cmd.CommandText = sql;
            List<TColumn> columnList = new List<TColumn>();
            SqlDataReader rdr = cmd.ExecuteReader();
            while (rdr.Read())
            {
                string colName = rdr.GetString(0);
                TColumn col = GetColumn(constraint.Table, colName);
                columnList.Add(col);
            }

            return columnList;
        }
}
}
