﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;
using System.IO;
using Microsoft.SqlServer.Management.Common;
using Microsoft.SqlServer.Management.Smo;
using FDAL;
using FDAL.Gen;
using FDAL.Database;

namespace KDAL.Desktop
{
    public partial class Form1 : Form
    {
        private SqlConnection connection;
        private string selectedDatabase;
        private string dalFolder;
       
        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            
        }

        private void Form1_Load(object sender, EventArgs e)
        {
        }

       

        private void btnConnect_Click(object sender, EventArgs e)
        {
            FrmConnection connectionForm = new FrmConnection();
            if (connectionForm.ShowDialog() == DialogResult.OK)
            {
                if (connection != null)
                {
                    if (connection.State == ConnectionState.Open)
                    {
                        connection.Close();
                    }
                }

                connection = connectionForm.Connection;
                selectedDatabase = connectionForm.cmbDatabase.Text;
                if (connection.State == ConnectionState.Open)
                {
                    //lblState.Text = string.Format("Connected to [{0}] On [{1}]", connectionForm.cmbDatabase.Text, connectionForm.cmbServer.Text);
                }
            }
        }

        private void lblState_Click(object sender, EventArgs e)
        {

        }

        private void btnEntities_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "C# Files (*.cs)|*.cs";
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                //lblEntities.Text = Path.GetFileName(dlg.FileName);
            }
        }

        private void updateDatabase()
        {
            if (connection == null || connection.State != ConnectionState.Open)
            {
                MessageBox.Show("No database is selected", "Failure", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            try
            {
                TDatabaseExplorer exp = new TDatabaseExplorer(connection);
                List<TTable> tables = exp.GetTables();

                TStoredProcGenerator gen = new TStoredProcGenerator(connection, selectedDatabase);
                string script = gen.Generate();

                Server server = new Server(new ServerConnection(connection));
                server.ConnectionContext.ExecuteNonQuery(script);
                MessageBox.Show("Database update successful", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Database update failed:" + ex.Message, "Failure", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void saveToFile(string file, string content)
        {
            FileStream fs = new FileStream(file, FileMode.Create);
            StreamWriter writer = new StreamWriter(fs);
            writer.Write(content);
            writer.Flush();
            writer.Close();
        }

        private void updateDalFiles()
        {
            if (dalFolder == null)
            {
                MessageBox.Show("No folder for DAL files is selected", "Failure", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            try
            {
                TDatabaseExplorer exp = new TDatabaseExplorer(connection);
                List<TTable> tables = exp.GetTables();

                TEntityGenerator entityGen = new TEntityGenerator(connection);
                string entitiesCode = entityGen.GenerateEntities(tables, "DAL");
                string entityFile = Path.Combine(dalFolder, "Entities.cs");
                saveToFile(entityFile, entitiesCode);

                TMethodGenerator methodGen = new TMethodGenerator(connection);
                string storeCode = methodGen.GenerateClass("DAL", "DataStore");
                string storeFile = Path.Combine(dalFolder, "DataStore.cs");
                saveToFile(storeFile, storeCode);

                MessageBox.Show("DAL files update successful", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception ex)
            {
                MessageBox.Show("DAL files update failed:" + ex.Message, "Failure", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (connection != null)
            {
                connection.Close();
            }
        }

        private void btnSelect_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog dlg = new FolderBrowserDialog();
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                dalFolder = dlg.SelectedPath;
            }
        }

        private void btnUpdateDAL_Click(object sender, EventArgs e)
        {
            updateDalFiles();
        }

        private void btnUpdateDatabase_Click(object sender, EventArgs e)
        {
            updateDatabase();
        }

        private void btnUpdateAll_Click(object sender, EventArgs e)
        {
            updateDatabase();
            updateDalFiles();
        }

       
        
    }
}
