﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;
using KDAL;
using FDAL;
using FDAL.Database;

namespace KDAL.Desktop
{
    public partial class FrmConnection : Form
    {
        private string connectionString;
        private SqlConnection connection;
        
        public FrmConnection()
        {
            InitializeComponent();
        }

        private void testConnection()
        {
            string err = null;
            bool succ = TDatabaseExplorer.TestConnection(getCN(), out err);
            if (succ)
            {
                MessageBox.Show("Connection Successful", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show(err, "Failure", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void updateDatabaseList()
        {
            string err = null;
            bool succ = TDatabaseExplorer.TestConnection(getCN(), out err);
            if (succ)
            {
                using (SqlConnection conn = new SqlConnection(getCN()))
                {
                    conn.Open();
                    TDatabaseExplorer exp = new TDatabaseExplorer(conn);
                    List<TDatabase> databases = exp.GetDatabases();
                    cmbDatabase.ValueMember = "Name";
                    cmbDatabase.DataSource = databases;
                }
            }
            else
            {
                cmbDatabase.DataSource = null;
            }
        }

        private bool prepareConnection()
        {
            bool result = false;
            string err = null;

            if (cmbDatabase.SelectedIndex == -1)
            {
                MessageBox.Show("You should select a database", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            
            bool succ = TDatabaseExplorer.TestConnection(getCN(), out err);
            if (succ)
            {
                connection = new SqlConnection(getCN());
                connection.Open();
                result = true;
            }
            else
            {
                MessageBox.Show(err, "Failure", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return result;
        }

        private string getCN()
        {
            string user = cmbAuth.SelectedIndex == 1 ? cmbUser.Text : null;
            string password = cmbAuth.SelectedIndex == 1 ? edtPassword.Text : null;
            string cn = TConnectionStringBuilder.GetConnectionString(cmbServer.Text, cmbDatabase.Text, user, password);
            return cn;
        }

        private void cmbAuth_SelectedIndexChanged(object sender, EventArgs e)
        {
            bool en = cmbAuth.SelectedIndex == 1;
            cmbUser.Enabled = en;
            edtPassword.Enabled = en;
            lblUser.Enabled = en;
            lblPassword.Enabled = en;
        }

        public string ConnectionString
        {
            get
            {
                return connectionString;
            }
        }

        public SqlConnection Connection
        {
            get
            {
                return connection;
            }
        }

        private void btnTest_Click(object sender, EventArgs e)
        {
            testConnection();
        }

        private void cmbDatabase_Enter(object sender, EventArgs e)
        {
            updateDatabaseList();
        }

        private void FrmConnection_Load(object sender, EventArgs e)
        {
            cmbAuth.SelectedIndex = 0;
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            bool succ = prepareConnection();
            if (succ)
            {
                this.DialogResult = DialogResult.OK;
                this.Close();
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }
    }
}
